<?php

/**
 * @file
 * Schedule the newsletter or preview and send it manually.  Provides statistics.
 */

/**
 * Updates the next date to send the newsletter.
 */
function _sunmailer_find_next_date(&$newsletter_schedule, $earliest_date) {
  $datetime = new DateTime(date('Y-m-d H:i', $newsletter_schedule->next_date));
  switch ($newsletter_schedule->type) {
  case 'none':
  case 'single':
    return TRUE;

  case 'interval':
    // Increment by the interval until we pass the earliest date.
    $increment = '+' . $newsletter_schedule->data . ' ' . $newsletter_schedule->unit;
    while ($datetime->format('U') < $earliest_date) {
      $datetime->modify($increment);
    }

    $newsletter_schedule->next_date = $datetime->format('U');
    return TRUE;

  case 'absolute':
    // Increment by the unit until we pass the earliest date.
    $increment = '+1 ' . $newsletter_schedule->unit;
    while ($datetime->format('U') < $earliest_date) {
      $datetime->modify($increment);
    }

    // Figure out the target unit (e.g.: month) and targets (e.g.: 1st and 15th days of the month).
    $index = strpos($newsletter_schedule->data, ':');
    $target_unit = substr($newsletter_schedule->data, 0, $index);
    $targets = explode(',', substr($newsletter_schedule->data, $index + 1));
    switch ($target_unit)
    {
    case 'hour':
      $target_format = 'i';
      break;
    case 'day':
      $target_format = 'G';
      break;
    case 'week':
      $target_format = 'w';
      break;
    case 'month':
      $target_format = 'j';
      break;
    default:
      return FALSE;
    }

    // Increment by the unit until we hit a target.
    while (!in_array($datetime->format($target_format), $targets))
    {
      $datetime->modify($increment);
    }

    $newsletter_schedule->next_date = $datetime->format('U');
    return TRUE;

  default:
    return FALSE;
  }
}

/**
 * Checks if the newsletter can be sent and finds the next scheduled date if so.
 */
function sunmailer_check_schedule($newsletter) {
   // Check if the newsletter can be sent.
  $newsletter_schedule = sunmailer_newsletter_schedule_load($newsletter->nlid);
  if ($newsletter_schedule->type == 'none')
  {
    return FALSE;
  }
  if ($newsletter_schedule->next_date > time())
  {
    return FALSE;
  }

  // Find the next scheduled date.
  if ($newsletter_schedule->type == 'single') {
    newsletter_schedule_delete($newsletter->nlid);
    return TRUE;
  }
  if (!_sunmailer_find_next_date($newsletter_schedule, time())) {
    return FALSE;
  }
  sunmailer_newsletter_schedule_save($newsletter_schedule);
  return TRUE;
}

/**
 * Checks the newsletter for errors.
 */
function sunmailer_check_newsletter() {
  module_load_install('sunmailer');
  module_load_include('inc', 'sunmailer', 'sunmailer.xml');
  $errors = array();
  // Check the requirements.
  $requirements = module_invoke('sunmailer', 'requirements', 'runtime');
  if (count($requirements)) {
    foreach ($requirements as $requirement) {
      $errors[] = $requirement['description'];
    }
    return $errors;
  }
  // Check the views.
  $sections = sunmailer_get_sections();
  views_get_all_views(); // http://drupal.org/node/670308
  foreach ($sections as $section) {
    $name = $section['view'];
    $display = $section['display'];
    $view = views_get_view($name);
    if (!$view) {
      $errors[] = t('View !name does not exist.', array('!name' => $name));
      continue;
    }
    if ($view->base_table != 'node') {
      $errors[] = t('View !name is not a node view.', array('!name' => $name));
      continue;
    }
    if ($view->disabled) {
      $errors[] = t('View !name is disabled.', array('!name' => $name));
      continue;
    }
    if (!array_key_exists($display, $view->display)) {
      $errors[] = t('View !name does not contain the display !display.', array('!name' => $name, '!display' => $display));
      continue;
    }
  }
  if (count($errors)) {
    return $errors;
  }
  // Check the XSL files.
  $xml = sunmailer_xml_newsletter(array_keys(sunmailer_get_sections()));
  if (!$xml) {
    return array();
  }
  $formats = sunmailer_get_formats();
  foreach ($formats as $format) {
    $row = db_fetch_object(db_query("SELECT filename, filepath FROM {files} WHERE fid = %d", $format['fid']));
    $xsl_doc = new DOMDocument();
    if (!$xsl_doc->load($row->filepath)) {
      $errors[] = t('!name is not a valid XSL file.', array('!name' => $row->filename));
      continue;
    }
    $xslt = new XSLTProcessor();
    $xslt->importStyleSheet($xsl_doc);
    if (!$xslt->transformToXml($xml)) {
      $errors[] = t('An XSLT error occurred when transforming the XML with !name.', array('!name' => $row->filename));
    }
  }
  return $errors;
}

/**
 * Mails the newsletter.
 */
function sunmailer_send_newsletter($newsletter) {
  // Check for errors.
  /*$errors = sunmailer_check_newsletter();
  if (count($errors)) {
    watchdog('mail', 'Error(s) sending the newsletter: ' . implode(' ', $errors), array(), WATCHDOG_ERROR);
    return;
  }*/

  // Check if the newsletter has any content.
  module_load_include('inc', 'sunmailer', 'sunmailer.xml');
  $xml = sunmailer_xml_newsletter($newsletter);
  if (!$xml) {
    watchdog('mail', 'Skipping the newsletter; it has no new content.');
    return;
  }

  // Perform time-based logic.
  variable_set('sunmailer_last_sent', time());
  if (variable_get('sunmailer_cutoff', 0)) {
    variable_set('sunmailer_cutoff', time());
  }

  // Log the beginning of the mailing.
  watchdog('mail', 'Sending the newsletter');
  $mailer = sunmailer_get_mailer();

  // Group subscribers by common subscriptions.
  $grouped_subscriptions = sunmailer_newsletter_grouped_subscriptions_load($newsletter);
  foreach ($grouped_subscriptions as $sid_list => $addresses) {
    $sids = explode(',', $sid_list);

    // Generate the newsletter body.
    $xml = sunmailer_xml_newsletter($newsletter, $sids);
    if (!$xml) {
      continue;
    }
    $body = sunmailer_transform_xml($xml, $newsletter->design);
    $alt_body = $newsletter->alt_design ? sunmailer_transform_xml($xml, $newsletter->alt_design) : NULL;

    // Send the newsletter.
    if (variable_get('sunmailer_max_recipients', 1) == 1) {
      foreach ($addresses as $address) {
        $subscriber_body = sunmailer_add_subscription_link($body, $address);
        $subscriber_alt_body = ($alt_body == NULL) ? NULL : sunmailer_add_subscription_link($alt_body, $address);
        sunmailer_send_mail($mailer, array($address), $newsletter->subject, $subscriber_body, $newsletter->design->html, $subscriber_alt_body);
      }
    }
    else {
      $body = sunmailer_add_subscription_link($body);
      $alt_body = ($alt_body == NULL) ? NULL : sunmailer_add_subscription_link($alt_body);
      sunmailer_send_mail($mailer, $addresses, $newsletter->subject, $body, $newsletter->design->html, $alt_body);
    }
  }

  // Log the end of the mailing.
  watchdog('mail', 'Sent the newsletter');
}

/**
 * Generates a form for newsletter settings.
 */
function sunmailer_mail_settings($form_state = NULL, $newsletter_settings) {
  $form = array();
  $form['nlid'] = array(
    '#type' => $value,
    '#value' => $newsletter_settings->nlid,
  );
  $form['cutoff'] = array(
    '#type' => 'fieldset',
    '#title' => 'Cutoff date',
    '#collapsible' => FALSE,
  );
  $form['cutoff']['use_cutoff'] = array(
    '#type' => 'checkbox',
    '#title' => t('Exclude content created before a specific date'),
    '#default_value' => $newsletter_settings->cutoff_date,
  );
  $form['cutoff']['cutoff_date'] = array(
    '#title' => t('Cutoff date'),
    '#required' => TRUE,
    '#default_value' => format_date($newsletter_settings->cutoff_date ? $newsletter_settings->cutoff_date : time(), 'custom', 'Y-m-d H:i'),
  );
  _sunmailer_set_date_type($form['cutoff']['cutoff_date']);
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#description' => t('(Optional) This message will be displayed in the header of the newsletter'),
    '#default_value' => '',
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Submission callback for newsletter settings form.
 */
function sunmailer_mail_settings_submit($form, &$form_state) {
  $newsletter_settings = sunmailer_newsletter_settings_load($nlid);
  $newsletter_settings->cutoff_date = $values['use_cutoff'] ? strtotime($values['cutoff_date']) : 0;
  $newsletter_settings->message = $values['message'];
  sunmailer_newsletter_settings_save($newsletter_settings);
}

/**
 * Generates a form to schedule the newsletter.
 */
function sunmailer_mail_schedule($form_state = NULL, $newsletter) {
  $form = array();
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $newsletter->nlid,
  );
  $newsletter_schedule = sunmailer_newsletter_schedule_load($newsletter->nlid);

  // Parse out the default schedule information.
  switch ($newsletter_schedule->type) {
  case 'none':
    $default_type = 'none';
    break;
  case 'single':
    $default_type = 'single';
    break;
  case 'interval':
    $default_type = $newsletter_schedule->unit . '_interval';
    $default_interval = $newsletter_schedule->data;
    break;
  case 'absolute':
    $index = strpos($newsletter_schedule->data, ':');
    $unit2 = substr($newsletter_schedule->data, 0, $index);
    $default_type = $newsletter_schedule->unit . '_' . $unit2;
    $default_selection = substr($newsletter_schedule->data, $index + 1);
    break;
  }

  // This part of the form will be customly themed.
  $form['schedule']['type'] = array(
    '#type' => 'radios',
    '#options' => array(
      'none' => 'No schedule',
      'single' => 'one time only',
      'minute_interval' => 'every !field minute(s)',
      'hour_interval' => 'every !field hour(s)',
      'day_interval' => 'every !field days(s)',
      'week_interval' => 'every !field week(s)',
      'month_interval' => 'every !field month(s)',
      'minute_hour' => 'on minute(s) !field of every hour (0-59)',
      'hour_day' => 'on hour(s) !field of every day (0-23)',
      'day_week' => 'on the following day(s) of every week:',
      'day_month' => 'on day(s) !field of every month (1-31)',
    ),
    '#default_value' => $default_type,
  );
  $types = array('minute_interval', 'hour_interval', 'day_interval', 'week_interval', 'month_interval');
  foreach ($types as $type) {
    $form['schedule'][$type] = array(
      '#type' => 'textfield',
      '#size' => 2,
      '#default_value' => ($type == $default_type) ? $default_interval : '',
    );
  }
  $types = array('minute_hour', 'hour_day', 'day_month');
  foreach ($types as $type) {
    $form['schedule'][$type] = array(
      '#type' => 'textfield',
      '#size' => 8,
      '#default_value' => ($type == $default_type) ? $default_selection : '',
    );
  }
  $form['schedule']['day_week'] = array(
    '#type' => 'checkboxes',
    '#options' => array(
      0 => t('Sunday'),
      1 => t('Monday'),
      2 => t('Tuesday'),
      3 => t('Wednesday'),
      4 => t('Thursday'),
      5 => t('Friday'),
      6 => t('Saturday'),
    ),
    '#default_value' => ('day_week' == $default_type) ? explode(',', $default_selection) : array(),
  );
  $form['schedule']['next_date'] = array(
    '#title' => t('Send the next newsletter no sooner than'),
    '#default_value' => format_date($newsletter_schedule->next_date ? $newsletter_schedule->next_date : time(), 'custom', 'Y-m-d H:i'),
  );
  _sunmailer_set_date_type($form['schedule']['next_date']);

  // Generate the rest of the form.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  return $form;
}

function sunmailer_mail_schedule_submit($form, &$form_state) {
  $values = $form_state['values'];
  $newsletter_schedule = sunmailer_newsletter_schedule_load($values['nlid']);

  // Check if a schedule exists.
  if ($values['type'] == 'none') {
    sunmailer_newsletter_schedule_delete($newsletter_schedule->nlid);
    return;
  }
  $newsletter_schedule->next_date = strtotime($values['next_date']);

  // Process a one-time schedule.
  if ($values['type'] == 'single') {
    $newsletter_schedule->type = 'single';
    sunmailer_newsletter_schedule_save($newsletter_schedule);
    return;
  }

  // Process interval schedules.
  if (strpos($values['type'], '_interval') !== FALSE) {
    $type_parts = explode('_', $values['type']);
    $newsletter_schedule->type = 'interval';
    $newsletter_schedule->unit = $type_parts[0];
    $newsletter_schedule->data = $values[$values['type']];
    sunmailer_newsletter_schedule_save($newsletter_schedule);
    return;
  }

  // Process absolute schedules.
  $type_parts = explode('_', $values['type']);
  $newsletter_schedule->type = 'absolute';
  $newsletter_schedule->unit = $type_parts[0];

  // Encode the absolue schedule data.
  // Example: week:1,3 => Monday and Wednesday every week
  if ($type_parts[1] == 'week') {
    $selection = implode(',', array_filter($values[$values['type']]));
  }
  else {
    $selection = implode(',', preg_split('/\\D+/', $values[$values['type']], NULL, PREG_SPLIT_NO_EMPTY));
  }
  $newsletter_schedule->data = $type_parts[1] . ':' . $selection;

  // Make sure the next date aligns with the schedule.
  _sunmailer_find_next_date($newsletter_schedule, $newsletter_schedule->next_date);

  sunmailer_newsletter_schedule_save($newsletter_schedule);
}

function _sunmailer_render_schedule_option($option, $field = NULL) {
  $option['#title'] = t($option['#title'], array('!field' => drupal_render($field)));
  return drupal_render($option);
}

function theme_sunmailer_mail_schedule($form) {
  $schedule = $form['schedule'];
  $type = $schedule['type'];
  drupal_add_css(drupal_get_path('module', 'sunmailer') . '/sunmailer.mail-schedule.css');
  $html = '';

  // Do not schedule the newsletter.
  $fieldset = array(
    '#title' => t('Do not schedule the newsletter to be sent'),
    '#collapsible' => FALSE,
    '#value' => _sunmailer_render_schedule_option($type['none']),
    '#attributes' => array(),
  );
  $html .= theme('fieldset', $fieldset);

  // Schedule the newsletter.
  $value = '';
  $value .= _sunmailer_render_schedule_option($type['single']);
  $value .= _sunmailer_render_schedule_option($type['minute_interval'], $schedule['minute_interval']);
  $value .= _sunmailer_render_schedule_option($type['hour_interval'], $schedule['hour_interval']);
  $value .= _sunmailer_render_schedule_option($type['day_interval'], $schedule['day_interval']);
  $value .= _sunmailer_render_schedule_option($type['week_interval'], $schedule['week_interval']);
  $value .= _sunmailer_render_schedule_option($type['month_interval'], $schedule['month_interval']);
  $value .= _sunmailer_render_schedule_option($type['minute_hour'], $schedule['minute_hour']);
  $value .= _sunmailer_render_schedule_option($type['hour_day'], $schedule['hour_day']);
  $value .= _sunmailer_render_schedule_option($type['day_week']);
  $value .= drupal_render($schedule['day_week']);
  $value .= _sunmailer_render_schedule_option($type['day_month'], $schedule['day_month']);
  $value .= drupal_render($schedule['next_date']);

  $fieldset = array(
    '#title' => t('Schedule the newsletter to be sent...'),
    '#collapisble' => FALSE,
    '#value' => $value,
    '#attributes' => array(),
  );
  $html .= theme('fieldset', $fieldset);

  // Render the rest of the form.
  unset($form['schedule']);
  $html .= drupal_render($form);
  return $html;
}

/**
 * Displays a preview of the newsletter.
 */
function sunmailer_newsletter_preview($xid) {
  module_load_include('inc', 'sunmailer', 'sunmailer.xml');
  $xml = sunmailer_xml_newsletter(array_keys(sunmailer_get_sections()));
  if (!$xml) {
    print t('The newsletter currently has no content');
    exit;
  }
  if (!$xid) {
    drupal_set_header('Content-Type: text/xml');
    print str_replace('!sunmailer_unsubscribe', sunmailer_unsubscribe_link(), $xml->saveXML());
    return;
  }
  $formats = sunmailer_get_formats();
  $format = $formats[$xid];
  $preview = sunmailer_transform_xml($xml, $format);
  $preview = str_replace('!sunmailer_unsubscribe', sunmailer_unsubscribe_link(), $preview);
  if (!$format['html']) {
    $preview = nl2br(htmlentities($preview));
  }
  print $preview;
}

/**
 * Checks the last time SunMailer was sent.
 */
function sunmailer_check_last_sent($notify) {
  $last_sent = variable_get('sunmailer_last_sent', 0);
  if (!$last_sent) {
    return TRUE;
  }
  $last_delta = time() - $last_sent;
  $delay = 60 * variable_get('sunmailer_delay', 0);
  if ($last_delta > $delay) {
    if ($notify) {
      drupal_set_message(t('An email was last sent @date.', array('@date' => date('l, F j, Y g:i:s A', $last_sent))));
    }
    return TRUE;
  }
  if ($notify) {
    $message = t('An email was sent @delta seconds ago.  Please wait @delay seconds before sending another.', array('@delta' => $last_delta, '@delay' => $delay));
    drupal_set_message($message, 'warning');
  }
  return FALSE;
}

/**
 * Generates a form to send the newsletter.
 */
function sunmailer_mail_send($form_state = NULL, $newsletter) {
  $form = array();
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $newsletter->nlid,
  );

  // Check for errors.
  /*$errors = sunmailer_check_newsletter();
  if (count($errors)) {
    drupal_set_title('Error');
    $html = '<ul><li>' . implode('</li><li>', $errors) . '</li></ul>';
    return array('error' => array('#value' => $html));
  }*/

  // Check if the newsletter has any content.
  module_load_include('inc', 'sunmailer', 'sunmailer.xml');
  $xml = sunmailer_xml_newsletter($newsletter, array_keys($newsletter->sections));
  if (!$xml) {
    $args = array(
      '!cutoff_url' => url("admin/settings/sunmailer/mail/{$newsletter->nid}/settings"),
      '@cutoff_date' => date('l, F j, Y g:i:s A', variable_get('sunmailer_last_sent', 0)),
    );
    $html = t('<p>The newsletter has no content. The <a href="!cutoff_url">cutoff date</a> is @cutoff_date.</p>', $args);
    return array('nocontent' => array('#value' => $html));
  }

  // Add preview links.
  // TODO

  // Save the header message or not.
  if (variable_get('sunmailer_message', '')) {
    $form['save_message'] = array(
      '#type' => 'checkbox',
      '#title' => t('Save the message in the header for future newsletters'),
      '#default_value' => TRUE,
    );
  }
  else {
    $form['save_message'] = array(
      '#type' => 'value',
      '#value' => TRUE,
    );
  }

  // Confirm the mailing.
  if (sunmailer_check_last_sent(TRUE)) {
    $form['override'] = array(
      '#type' => 'value',
      '#value' => FALSE,
    );
  }
  else {
    $form['override'] = array(
      '#type' => 'checkbox',
      '#title' => t('Override the time limit to send another email'),
    );
  }
  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm that you want to send the newsletter'),
  );

  // Submit the form.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Send'),
  );
  return $form;
}

/**
 * Validation callback for form to send the newsletter.
 */
function sunmailer_mail_send_validate($form, &$form_state) {
  $values = $form_state['values'];
  if (!$values['confirm']) {
    form_set_error('confirm', t('Cannot send without confirmation.'));
  }
  if (!sunmailer_check_last_sent(FALSE) && !$values['override']) {
    form_set_error('override', t('An email has been sent recently.'));
  }
}

/**
 * Submission callback for form to send the newsletter.
 */
function sunmailer_mail_send_submit($form, &$form_state) {
  $values = $form_state['values'];
  if ($values['override']) {
    watchdog('mail', 'Overriding SunMailer\'s time limit for sending another email');
  }
  sunmailer_send_newsletter(sunmailer_newsletter_load($values['nlid']));
  if (!$values['save_message']) {
    variable_del('sunmailer_message');
  }
  $form_state['redirect'] = user_access('access site reports') ? 'admin/reports/dblog' : 'admin/settings/sunmailer';
}

/**
 * Generates a form to send a custom message.
 */
function sunmailer_mail_custom(&$form_state = NULL) {
  $form = array();

  // Create the custom message.
  if (isset($form_state['storage']['preview'])) {
    $form['#prefix'] = $form_state['storage']['preview'];
  }
  $form['message'] = array(
    '#type' => 'fieldset',
    '#title' => t('Custom message'),
    '#description' => t('Message to email to all subscribers'),
  );
  $form['message']['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#required' => TRUE,
    '#default_value' => isset($form_state) ? $form_state['values']['subject'] : '',
  );
  $form['message']['body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#description' => t('HTML tags will be stripped from the alternate text version'),
    '#required' => TRUE,
    '#default_value' => isset($form_state) ? $form_state['values']['body'] : '',
  );

  // Confirm the mailing.
  if (sunmailer_check_last_sent(TRUE)) {
    $form['override'] = array(
      '#type' => 'value',
      '#value' => FALSE,
    );
  }
  else
  {
    $form['override'] = array(
      '#type' => 'checkbox',
      '#title' => t('Override the time limit to send another email'),
    );
  }
  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm that you want to send this email'),
    '#default_value' => FALSE,
  );

  // Submit the form.
  $form['preview'] = array(
    '#type' => 'submit',
    '#value' => t('Preview'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Send'),
  );
  return $form;
}

/**
 * Validation callback for the form to send a custom email.
 */
function sunmailer_mail_custom_validate($form, &$form_state) {
  $values = $form_state['values'];
  if ($form_state['clicked_button']['#value'] != t('Preview')) {
    if (!$values['confirm']) {
      form_set_error('confirm', t('Cannot send without confirmation.'));
    }
    if (!sunmailer_check_last_sent(FALSE) && !$values['override']) {
      form_set_error('override', t('An email has been sent recently.'));
    }
  }
}

/**
 * Submission callback for the form to send a custom message.
 */
function sunmailer_mail_custom_submit($form, &$form_state) {
  $values = $form_state['values'];
  $subject = $values['subject'];
  $body = filter_xss_admin($values['body']);
  $alt_body = strip_tags($body);

  if ($form_state['clicked_button']['#value'] == t('Send')) {
    // Perform time-based logic.
    variable_set('sunmailer_last_sent', time());
    if ($values['override']) {
      watchdog('mail', 'Overriding SunMailer\'s time limit for sending another email');
    }

    // Send the message.
    watchdog('mail', 'Sending a custom email message');
    $mailer = sunmailer_get_mailer();
    /*$addresses = sunmailer_get_all_subscribers();
    sunmailer_send_mail($mailer, $addresses, $subject, $body, TRUE, $alt_body);*/
    watchdog('mail', 'Sent a custom email message');

    // Redirect.
    unset($form_state['storage']);
    $form_state['redirect'] = user_access('access site reports') ? 'admin/reports/dblog' : 'admin/settings/sunmailer';
  }
  else {
     // Store a preview of the email newsletter.
     $preview  = "<h3>$subjct</h3><ul>";
     $preview .= "<li><strong>HTML:</strong> $body</li>";
     $preview .= "<li><strong>Text:</strong> $alt_body</li>";
     $preview .= "</ul>";
     $form_state['storage']['preview'] = $preview;
  }
}

/**
 * Calculates subscription statistics.
 */
function get_sunmailer_stats() {
  $sections = sunmailer_get_sections();
  $formats = sunmailer_get_formats();
  // Initialize the statistics.
  $stats['total'] = array('count' => 0, 'list' => array());
  $stats['splits']['section'] = array();
  foreach ($sections as $section) {
    $stats['splits']['section'][$section['name']] = array('count' => 0, 'list' => array());
  }
  $stats['splits']['format'] = array();
  foreach ($formats as $format) {
    $stats['splits']['format'][$format['name']] = array('count' => 0, 'list' => array());
  }
  // Calculate the statistics.
  $subscriptions = sunmailer_get_subscriptions();
  foreach ($subscriptions as $sid_list => $sections_group) {
    $sids = explode(',', $sid_list);
    foreach ($sections_group as $xid => $format_group) {
      $count = count($format_group);
      // Add to total list.
      $stats['total']['count'] += count($format_group);
      $stats['total']['list'] += $format_group;
      // Add to format list.
      $stats['splits']['format'][$formats[$xid]['name']]['count']  += $count;
      $stats['splits']['format'][$formats[$xid]['name']]['list']  += $format_group;
      // Add to section list.
      foreach ($sids as $sid) {
        $stats['splits']['section'][$sections[$sid]['name']]['count']  += $count;
        $stats['splits']['section'][$sections[$sid]['name']]['list']  += $format_group;
      }
    }
  }
  // Sort email addresses.
  sort($stats['total']['list']);
  foreach($stats['splits']['format'] as $name => &$group) {
    sort($group['list']);
  }
  foreach($stats['splits']['section'] as $name => &$group) {
    sort($group['list']);
  }
  return $stats;
}

/**
 * Displays a page of statistics.
 */
function sunmailer_stats_page() {
  return theme('sunmailer_stats', get_sunmailer_stats());
}

/**
 * Generates a collapsible list of email addresses.
 */
function _sunmailer_email_list($title, $group) {
  $markup = '';
  foreach ($group as $email) {
    $markup .= '<li>' . check_plain($email) . '</li>';
  }
  $list = array(
    '#type' => 'fieldset',
    '#title' => $title,
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $list['emails'] = array(
    '#type' => 'markup',
    '#prefix' => '<ul>',
    '#value' => $markup,
    '#suffix' => '</ul>',
  );
  return drupal_render($list);
}

/**
 * Default theme for statistics.
 */
function theme_sunmailer_stats($stats) {
  $html = '<h3>' . t('Total') . '</h3>';
  $title = t('Subscribers: @count', array('@count' => $stats['total']['count']));
  $html .= _sunmailer_email_list($title, $stats['total']['list']);
  foreach ($stats['splits'] as $type => $split) {
    $html .= '<h4>' . t('By @type', array('@type' => $type)) . '</h4>';
    foreach ($split as $name => $group) {
      $title = t('@name: @count', array('@name' => $name, '@count' => $group['count']));
      $html .= _sunmailer_email_list($title, $group['list']);
    }
  }
  return $html;
}
