<?php

/**
 * Builds a form for subscription settings.
 */
function sunmailer_subscribe_settings() {
  $form = array();

  // Display user registration settings.
  $form['register'] = array(
    '#type' => 'fieldset',
    '#title' => 'User registration settings',
  );
  $form['register']['subscribe_register'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display the subscription form on the user registration page'),
    '#default_value' => variable_get('sunmailer_subscribe_register', TRUE),
  );
  $form['register']['subscribe_register_weight'] = array(
    '#type' => 'weight',
    '#title' => t('Subscription form weight'),
    '#description' => t('Position the subscription form on the user registration page'),
    '#default_value' => variable_get('sunmailer_subscribe_register_weight', 5),
  );

  // Display security and privacy settings.
  $form['security'] = array(
    '#type' => 'fieldset',
    '#title' => t('Security and privacy settings'),
  );
  $form['security']['subscribe_authenticate'] = array(
    '#type' => 'radios',
    '#title' => t('Authentication settings'),
    '#description' => t('Authentication verifies that the user owns the e-mail address listed in the subscription form'),
    '#options' => array(
      'all' => t('(Recommended) Require authentication to edit a subscription or view the current subscription'),
      'view' => t('Do not require authentication to edit a subscription but require it to view the current subscription'),
      'none' => t('Do not require authentication to edit a subscription or view the current subscription'),
    ),
    '#default_value' => variable_get('sunmailer_subscribe_authenticate', 'all'),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Submission callback for the subscription settings callback.
 */
function sunmailer_subscribe_settings_submit($form, &$form_state) {
  $values = $form_state['values'];
  variable_set('sunmailer_subscribe_register', (bool)$values['subscribe_register']);
  variable_set('sunmailer_subscribe_register_weight', (int)$values['subscribe_register_weight']);
  variable_set('sunmailer_subscribe_authenticate', $values['subscribe_authenticate']);
}

/**
 * Builds the common part of all subscription forms.
 */
function sunmailer_subscriber_base_form($newsletters, $subscriber = NULL, $view_subscription = FALSE) {
  if (!is_array($newsletters)) {
    $newsletters = array($newsletters->nlid => $newsletters);
  }
  $form = array();
  $form['#tree'] = TRUE;

  // Build the e-mail address field.
  if ($subscriber) {
    $form['mail'] = array(
      '#type' => 'hidden',
      '#value' => $subscriber->mail,
    );
    $form['mail_display'] = array(
      '#type' => 'textfield',
      '#title' => t('E-mail adddress'),
      '#default_value' => $subscriber->mail,
      '#disabled' => TRUE,
    );
  }
  else {
    $form['mail'] = array(
      '#type' => 'textfield',
      '#title' => t('E-mail adddress'),
      '#required' => TRUE,
    );
  }

  // Build the subscription fields.
  $nlids = array();
  foreach ($newsletters as $nlid => $newsletter)
  {
    // Add the basic subscription checkbox if the newsletter has any sections.
    if (!count($newsletter->sections)) {
      continue;
    }

    $nlids[] = $nlid;
    $form[$nlid] = array(
      '#type' => 'fieldset',
      '#title' => $newsletter->title,
    );
    $form[$nlid]['subscribe'] = array(
      '#type' => 'checkbox',
      '#title' => 'Subscribe',
      '#default_value' => $view_subscription ? isset($subscriber->subscription[$nlid]) : FALSE,
    );

    // For multi-section newsletters, add an option to subscribe to individual sections.
    $section_options = array();
    foreach ($newsletter->sections as $sid => $section) {
      $section_options[$sid] = $section->title;
    }
    $default_sections = $form[$nlid]['subscribe']['#default_value'] ? $subscriber->subscription[$nlid] : array();
    $default_sections = (count($newsletter->sections) == count($default_sections)) ? array() : $default_sections;

    if (count($newsletter->sections) > 1) {
      $form[$nlid]['options'] = array(
        '#type' => 'fieldset',
        '#title' => t('Subscription options'),
        '#collapsible' => TRUE,
        '#collapsed' => !count($default_sections),
      );
      $form[$nlid]['options']['sections'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Sections'),
        '#description' => t('Leave blank to subscribe to all sections'),
        '#options' => $section_options,
        '#default_value' => $default_sections,
      );
    }
    else {
      $form[$nlid]['options']['sections'] = array(
        '#type' => 'hidden',
        '#value' => array_fill_keys(array_keys($section_options), 0),
      );
    }
  }

  $form['nlids'] = array(
    '#type' => 'value',
    '#value' => $nlids,
  );

  return $form;
}

function sunmailer_subscriber_base_form_submit($edit, $authenticated = FALSE) {
  // Update the subscription.
  $subscriber = sunmailer_subscriber_load($edit['mail']);
  $nlids = $edit['nlids'];
  $newsletters = sunmailer_newsletter_load_multiple($nlids, TRUE);
  foreach ($nlids as $nlid)
  {
    $newsletter = $newsletters[$nlid];
    if ($edit[$nlid]['subscribe']) {
      $sids = array_keys(array_filter($edit[$nlid]['options']['sections']));
      $subscriber->subscription[$nlid] = count($sids) ? $sids : array_keys($newsletter->sections);
    }
    else {
      unset($subscriber->subscription[$nlid]);
    }
  }

  // Save these changes or send a confirmation email.
  if ($authenticated) {
    sunmailer_subscriber_save($subscriber);
  }
  else {
    drupal_set_message("TODO - send confirmation email.");
  }
}

/**
 * Authenticates a subscriber using a hash based on their e-mail address.
 */
function sunmailer_authenticate_subscriber($mail, $hash) {
  return $hash == md5($mail . drupal_get_private_key());
}

/**
 * Helper function to build the main subscription form.
 */
function _sunmailer_subscriber_form($newsletters, $save, $subscriber = NULL, $view_subscription = FALSE) {
  $form = sunmailer_subscriber_base_form($newsletters, $subscriber, $view_subscription);
  if ($save) {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save changes'),
    );
  }
  else {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Request changes'),
    );
  }
  return $form;
}

/**
 * Submits an automated subscription request.
 */
function sunmailer_autosubscribe($mail, $nlids, $sids) {
  $edit['mail'] = $mail;
  $edit['nlids'] = array();
  $newsletters = sunmailer_newsletter_load_multiple($nlids, TRUE);
  foreach ($newsletters as $nlid => $newsletter) {
    if (!count($newsletter->sections)) {
      continue;
    }
    $edit['nlids'][] = $nlid;
    foreach ($newsletter->sections as $sid => $section) {
      $edit[$nlid]['options']['sections'][$sid] = in_array($sid, $sids) ? $sid : 0;
    }
    $edit[$nlid]['subscribe'] = count(array_filter($edit[$nlid]['options']['sections']));
  }
  sunmailer_subscriber_base_form_submit($edit, TRUE);
}

/**
 * Builds the main subscription form.
 */
function sunmailer_subscriber_form($form_state = NULL, $newsletters) {
  global $user;

  // Handle an automated subscription.
  if (isset($form_state['autosubscribe']) && $form_state['autosubscribe']) {
     $newsletters = $form_state['newsletters'];
     $subscriber = sunmailer_subscriber_load($form_state['mail']);
     return _sunmailer_subscriber_form($newsletters, TRUE, $subscriber, FALSE);
  }

  // First check for a e-mail address.
  $mail = isset($_GET['mail']) ? $_GET['mail'] : '';
  if ($mail) {

    // Load and authenticate the subscriber.
    $subscriber = sunmailer_subscriber_load($mail);
    $authenticated = FALSE;
    $hash = isset($_GET['hash']) ? $_GET['hash'] : '';
    if ($hash && sunmailer_authenticate_subscriber($mail, $hash)) {
      $authenticated = TRUE;
    }
    else if ($user->uid && ($mail == $user->mail)) {
      $authenticated = TRUE;
    }

    // Check if an automated subscription should be performed.
    $nlids = isset($_GET['nlids']) ? explode(',', $_GET['nlids']) : array();
    if ($nlids && $authenticated) {
      $sids = isset($_GET['sids']) ? explode(',', $_GET['sids']) : array();
      sunmailer_autosubscribe($mail, $nlids, $sids);
      return;
    }

    // Build the subscription form.
    switch (variable_get('sunmailer_subscribe_authenticate', 'all')) {
    case 'all':
    default:
      return _sunmailer_subscriber_form($newsletters, $authenticated, $subscriber, $authenticated);
    case 'view':
      return _sunmailer_subscriber_form($newsletters, TRUE, $subscriber, $authenticated);
    case 'none':
      return _sunmailer_subscriber_form($newsletters, TRUE, $subscriber, TRUE);
    }
  }

  // Next, check for an authenticated user.
  if ($user->uid) {
    $subscriber = sunmailer_subscriber_load($user->mail);
    return _sunmailer_subscriber_form($newsletters, TRUE, $subscriber, TRUE);
  }

  // Display the subscription form for an anonymous user.
  switch (variable_get('sunmailer_subscribe_authenticate', 'all')) {
  case 'all':
  default:
    $save = FALSE;
    break;
  case 'view':
  case 'none':
    $save = TRUE;
    break;
  }
  return _sunmailer_subscriber_form($newsletters, $save);
}

function sunmailer_subscriber_form_submit($form, &$form_state) {
  $authenticated = $form_state['clicked_button']['#value'] == t('Save changes');
  sunmailer_subscriber_base_form_submit($form_state['values'], TRUE);
}