<?php

/**
 * @file
 * Builds the XML newsletter and the XSL to transform it.
 */

/**
 * Generates the XML for a newsletter with selected sections.
 */
function sunmailer_xml_newsletter($newsletter, $sids = array(), $account = NULL, $reset = FALSE) {
  // Cache all newsletter content for anonymous users.
  static $cached = FALSE;
  static $cached_sections = array();
  static $no_section_title = FALSE;
  if (!$cached || $reset) {
    $sids = array_keys($newsletter->sections);
    $cached_sections = sunmailer_newsletter_content_load($newsletter, $sids, user_load(0));
    $no_section_title = count($sids) == 1;
    $cached = TRUE;
  }

  // Load the newsletter content.
  if (!$account) {
    $account = user_load(0);
  }
  if (!count($sids))
  {
    $sids = array_keys($newsletter->sections);
  }
  if ($account->uid) {
    $sections = sunmailer_newsletter_content_load($newsletter, $sids, $account);
  }
  else {
    $sections = array();
    foreach ($cached_sections as $sid => $cached_section) {
      if (in_array($sid, $sids)) {
        $sections[$sid] = $cached_section;
      }
    }
  }

  // Build the newsletter.
  $dom = new DOMDocument('1.0', variable_get('sunmailer_char_set', 'utf-8'));
  $newsletter_node = $dom->createElement('newsletter');
  $dom->appendChild($newsletter_node);
  sunmailer_xml_header($dom, $newsletter_node, $newsletter);
  $num_sections = 0;
  foreach ($sections as $sid => $section) {
    $section_node = sunmailer_xml_section($dom, $section, $no_section_title, $account);
    if ($section_node) {
      $num_sections++;
      $newsletter_node->appendChild($section_node);
    }
  }
  sunmailer_xml_footer($dom, $newsletter_node);

  // Check for an empty newsletter.
  if (!$num_sections) {
    $newsletter_node = NULL;
  }

  // Allow developers to alter the newsletter XML.
  module_invoke_all('sunmailer_xml_newsletter_alter', $dom, $newsletter_node, $account);
  return $dom;
}

/**
 * Loads the content for one or more sections of a newsletter.
 */
function sunmailer_newsletter_content_load($newsletter, $sids, $account) {
  // Impersonate the account.
  global $user;
  $original_user = $user;
  session_save_session(FALSE);
  $user = $account;

  // Load the content for the specified sections.
  $sections = $newsletter->sections;
  foreach ($sections as $sid => $section) {
    if (!in_array($sid, $sids)) {
      unset($sections[$sid]);
    }
  }
  foreach ($sections as &$section) {
    sunmailer_section_content_load($section, $account);
  }

  // Revert to the original user.
  $user = $original_user;
  session_save_session(TRUE);
  return $sections;
}

/**
 * Load the content for a section.
 */
function sunmailer_section_content_load(&$section, $account) {
  global $base_root;

  // Load the view's content.
  $view = views_get_view($section->view);
  if ($section->args) {
    $args = explode('/', str_replace('!uid', $account->uid, $section->args));
  }
  else {
    $args = array();
  }
  $view->execute_display($section->display, $args);

  // Add the view's content to the section.
  $section->nodes = array();
  foreach ($view->result as $row_key => $row) {
    // Check if the node should be excluded.
    if (!$row->nid) {
      continue;
    }
    $node_fields = array();
    $node = node_load($row->nid);
    if ($node->created < variable_get('sunmailer_cutoff', 0)) {
      continue;
    }

    // Load the node and all fields provided by the view.
    $node_fields['node'] = $node;
    foreach ($view->field as $field => $v) {
      $value = $view->render_field($field, $row_key);
      if ($value) {
        $node_fields[$field] = $value;
      }
    }
    $section->nodes[] = $node_fields;
  }
}

/**
 * Generates the XML for the header.
 */
function sunmailer_xml_header(&$dom, &$newsletter_node, $newsletter) {
  // Retrieve the header information.
  $title = filter_xss($newsletter->title);
  $path = '';
  if ($newsletter->logo_fid) {
    $path = db_result(db_query("SELECT filepath FROM {files} WHERE fid = %d", $newsletter->logo_fid));
  }
  $logo = $path ? url($path, array('absolute' => TRUE)) : '';
  $link = url('<front>', array('absolute' => TRUE));
  if ($newsletter->date_format) {
    $date = filter_xss(date($newsletter->date_format));
  }
  $message = filter_xss_admin($newsletter->message);

  // Build the header XML.
  _sunmailer_append_cdata_child($dom, $newsletter_node, 'title', $title);
  if ($logo) {
    _sunmailer_append_cdata_child($dom, $newsletter_node, 'logo', $logo);
  }
  _sunmailer_append_cdata_child($dom, $newsletter_node, 'link', $link);
  if ($date) {
    _sunmailer_append_cdata_child($dom, $newsletter_node, 'date', $date);
  }
  if ($message) {
    _sunmailer_append_cdata_child($dom, $newsletter_node, 'message', $message);
  }
  return;
}

/**
 * Generates the XML for a section.
 */
function sunmailer_xml_section(&$dom, $section, $no_title, $account) {
  // Build the section XML.
  $section_node = $dom->createElement('section');
  if (!$no_title) {
    _sunmailer_append_cdata_child($dom, $section_node, 'title', filter_xss($section->title));
  }
  foreach ($section->nodes as $node_fields) {
    $section_node->appendChild(sunmailer_xml_content($dom, $node_fields, $account));
  }

  // Check for an empty section.
  if (!count($section->nodes)) {
    $section_node = NULL;
  }

  // Allow developers to alter the section XML.
  module_invoke_all('sunmailer_xml_section_alter', $dom, $section_node, $section, $account);
  return $section_node;
}

/**
 * Generates the XML for a content piece.
 */
function sunmailer_xml_content(&$dom, $node_fields, $account) {
  // Retrieve the content information.
  $node = $node_fields['node'];
  $title = $node->title;
  $link = url("node/{$node->nid}", array('absolute' => TRUE));
  if ($node->uid) {
    $author = $node->name;
    $authorlink = url("user/{$node->uid}", array('absolute' => TRUE));
  }
  $date = format_date($node->created);
  $teaser = $node->teaser;

  // Add basic node information to the content XML.
  $content_node = $dom->createElement('content');
  _sunmailer_append_cdata_child($dom, $content_node, 'title', $title);
  _sunmailer_append_cdata_child($dom, $content_node, 'link', $link);
  if ($author) {
    _sunmailer_append_cdata_child($dom, $content_node, 'author', $author);
    _sunmailer_append_cdata_child($dom, $content_node, 'authorlink', $authorlink);
  }
  _sunmailer_append_cdata_child($dom, $content_node, 'date', $date);
  _sunmailer_append_cdata_child($dom, $content_node, 'teaser', $teaser);

  // Add additional fields to the content XML.
  foreach ($node_fields as $name => $field) {
    if ($name == 'node') {
      continue;
    }
    _sunmailer_append_cdata_child($dom, $content_node, $name, $field);
  }

  // Allow developers to alter the content XML.
  module_invoke_all('sunmailer_xml_content_alter', $dom, $content_node, $node_fields, $account);
  return $content_node;
}

/**
 * Generates the XML for the footer.
 */
function sunmailer_xml_footer(&$dom, &$newsletter_node) {
  _sunmailer_append_cdata_child($dom, $newsletter_node, 'subscription-link', '!sunmailer_subscription');
  return;
}

/**
 * Adds an XML element with its data enclosed in a CDATA section.
 */
function _sunmailer_append_cdata_child(&$dom, &$parent_node, $title, $data) {
  $child_node = $dom->createElement($title);
  $child_node->appendChild($dom->createCDATASection($data));
  $parent_node->appendChild($child_node);
}

/**
 * Transforms XML into text or HTML.
 */
function sunmailer_transform_xml($xml, $design) {
  // Load the XSL.
  $doc = new DOMDocument();
  $doc->load($design->xsl_path, LIBXML_NOCDATA);
  $xslt = new XSLTProcessor();
  $xslt->importStyleSheet($doc);

  // Create a newsletter with only absolute URLs.
  $content = $xslt->transformToXML($xml);
  global $base_url;
  $content = preg_replace('|href="/|', "href=\"$base_url/", $content);

  // Do additional processing for text newsletters.
  if (!$design->html) {
    $content = strip_tags($content);
    $content = html_entity_decode($content);
  }
  return $content;
}

/**
 * Inserts the subscription link into the newsletter.
 */
function sunmailer_add_subscription_link($content, $mail = NULL)
{
  $link = sunmailer_subscription_link($mail);
  return str_replace('!sunmailer_subscription', $link, $content);
}